@@ -47,8 +47,8 @@
 #' The \code{stength_sep} argument is \code{NULL} by default, but can be used to
 #' identify shorthand for morning and evening doses. For example, consider the
 #' phrase \sQuote{Lamotrigine 300-200} (meaning 300 mg in the morning and 200 mg
-#' in the evening). The argument \code{strength_sep = '-'} can identify both
-#' \emph{300} and \emph{200} as \emph{dose} in this phrase.
+#' in the evening). The argument \code{strength_sep = '-'} identifies
+#' the full expression \emph{300-200} as \emph{dose} in this phrase.
 #'
 #' By default, the \code{drug_list} argument is \dQuote{rxnorm} which calls \code{data(rxnorm_druglist)}.
 #' A custom drug list in the form of a character string can be supplied instead, or can be appended
@@ -86,23 +86,16 @@
 #' medExtractR(note2, c("lamotrigine", "ltg"), 130, "mg", 1, strength_sep = "-")
 #' }
 
-medExtractR <- function(note,
-                          drug_names,
-                          window_length,
-                          unit,
-                          max_dist = 0,
-                          drug_list = "rxnorm",
-                          lastdose = FALSE,
-                          lastdose_window_ext = 1.5,
-                          strength_sep = NULL,
-                          flag_window = 30,
-                          dosechange_dict = NULL, ...) {
+medExtractR <- function(note, drug_names, window_length, unit, max_dist = 0,
+                        drug_list = "rxnorm", lastdose = FALSE, lastdose_window_ext = 1.5,
+                        strength_sep = NULL, flag_window = 30, dosechange_dict = 'default', 
+                        ...) {
   def.saf <- getOption('stringsAsFactors')
   on.exit(options(stringsAsFactors = def.saf))
   options(stringsAsFactors = FALSE)
-  if(is.null(dosechange_dict)) {
+  if(length(dosechange_dict) == 1 && dosechange_dict == 'default') {
     e <- new.env()
-    data("dosechange_vals", envir = e)
+    data("dosechange_vals", package = 'medExtractR', envir = e)
     dosechange_dict <- get("dosechange_vals", envir = e)
   }
 
@@ -143,6 +136,23 @@ medExtractR <- function(note,
         # need to make sure position is relative to full note
         pos <- is_match[[1]][1] + last_pos - 1
         match_info <- rbind(match_info, c(drug_match, pos, len))
+
+        # include misspelling in between current and previous, if it exists
+        if(nchar(drug)>3){
+          im <- utils::aregexec(paste0(drug, "\\b"),
+                                substr(current_string, 1, current_pos),
+                                max.distance = ifelse(nchar(drug) <=
+                                                        5, 1, max_dist), ignore.case = TRUE, fixed = FALSE)
+          if(im > 0){
+            l <- attributes(im[[1]])$match.length
+
+            match_info <- rbind(match_info, c(substr(current_string,
+                                                     start = im[[1]][1],
+                                                     stop = im[[1]][1] + l - 1),
+                                              im[[1]][1] + last_pos - 1, l))
+          }
+        }
+
       }
 
       if(!exists("pos")){pos <- 0}
@@ -176,8 +186,8 @@ medExtractR <- function(note,
   mi <- match_info[order(match_info[,'start_pos']),]
   lmi <- split(mi, mi[,'start_pos'])
   match_info <- do.call(rbind, lapply(lmi, function(dd) {
-      ddl <- dd[,'length']
-      dd[ddl == max(ddl),,drop = FALSE]
+    ddl <- dd[,'length']
+    dd[ddl == max(ddl),,drop = FALSE]
   }))
 
   nr <- nrow(match_info)
@@ -248,7 +258,7 @@ medExtractR <- function(note,
     drg <- drug_window$drug[i]
     drg_wndw <- drug_window$window[i]
 
-    after_drg <- substr(drg_wndw, nchar(drg)+1, nchar(wndw))
+    after_drg <- substr(drg_wndw, nchar(drg)+1, nchar(drg_wndw))
     if(grepl(drg, after_drg, ignore.case = T)){
       return(paste0(drg, sub(paste0(drg, ".+"), "", after_drg)))
     }else{return(drg_wndw)}
@@ -265,7 +275,7 @@ medExtractR <- function(note,
   if("rxnorm" %in% drug_list) {
     # default - using rxnorm drug list
     e <- new.env()
-    data("rxnorm_druglist", envir = e)
+    data("rxnorm_druglist", package = 'medExtractR', envir = e)
     dl <- get("rxnorm_druglist", envir = e)
 
     # add additional terms if specified
@@ -277,7 +287,11 @@ medExtractR <- function(note,
     dl <- drug_list
   }
   # make sure drug names of interest are not in list
-  dl_lc <- setdiff(tolower(dl), tolower(drug_names))
+  rm_index <- unique(unlist(sapply(tolower(drug_names), function(x){
+    grep(x, tolower(dl))
+  })))
+  dl_lc <- tolower(dl[-rm_index])
+
   dl_wb <- paste0("\\b", dl_lc, "\\b")
   wndw_lc <- tolower(drug_window[,'window'])
   other_drug_list <- lapply(dl_wb, regexec, text=wndw_lc)
@@ -319,6 +333,7 @@ medExtractR <- function(note,
     rdf <- extract_entities(phrase = drug_window$window[i], p_start = drug_window$drug_start[i],
                             p_stop = drug_window$drug_stop[i], unit = unit,
                             strength_sep = strength_sep, ...)
+
     rdf <- rdf[!is.na(rdf[,'expr']),]
 
     # Extract last dose time if desired
@@ -342,17 +357,7 @@ medExtractR <- function(note,
       }
     }
 
-    if(nrow(rdf)==0){
-      if(drug_window$keep_drug[i]){
-        data.frame("entity" = 'DrugName',
-                   "expr" = paste(drug_window$drug[i],
-                                  paste(drug_window$drug_start[i],
-                                        drug_window$drug_start[i] + nchar(drug_window$drug[i]),
-                                        sep = ":"),
-                                  sep = ";"))
-      }
-      return(NA)
-    }
+    if(nrow(rdf)==0){return(NA)}
 
     rdf[nrow(rdf)+1,] <- c("DrugName", paste(drug_window$drug[i],
                                              paste(drug_window$drug_start[i],
@@ -389,5 +394,15 @@ medExtractR <- function(note,
   results <- unique(results[order(sp),])
   row.names(results) <- NULL
 
+  # sometimes same expression gets extracted as strength and dose due to second drug name cutting off doseamt
+  # check/correct for this here
+  ix <- names(which(table(results$pos)>1))
+  for(x in ix){
+    rs <- results[results$pos==x,]
+    if(all(sort(rs$entity)==c("Dose", "Strength"))){
+      results <- results[-which(results$entity=="Dose" & results$pos==x),]
+    }
+  }
+
   return(results)
 }
